const { ipcRenderer } = require('electron')
const sip = require("electron_baresip")
const { SipPhone } = require('webitel-sdk')

window.addEventListener('DOMContentLoaded', () => {
    if(window.$store){
        subscribeMutation(window.$store)
        subscribeAction(window.$store)
        checkActiveTask()
    }
    localStorage.setItem('auto-refresh', 5000)
    ipcRenderer.send("hide-disconnect-popup")    
})

window.addEventListener('keyup', async (e) => {
    if (e.keyCode === 123) {
        ipcRenderer.send('open-close-DevTools')
    }
    if (e.keyCode === 116) {
        //await unrg(window.cli, true)        
    }
}, true)

async  function unrg(cli, reload) {
    if (cli.phone) {
        await cli.phone.unregister();
        if(cli.phone.destroy)
            await cli.phone.destroy();
        cli.phone = null;
    }
    if(reload) {
        ipcRenderer.send('reload-page')
    }
    return;
}

window.onbeforeunload = () => {
    destroyNotification()
}

ipcRenderer.on('from-main', (event, err = '') => {
    alert(err)
    console.error(err)
})

ipcRenderer.on('change-status', (event, status = '') => {
    switch (status) {
        case 'pause':
            $store.dispatch('status/SET_AGENT_PAUSE_STATUS')
            break
        case 'online':
            $store.dispatch('status/SET_AGENT_WAITING_STATUS')
            break
        case 'offline':
            $store.dispatch('status/AGENT_LOGOUT')
            break
        default:
            console.error(status)
    }
})

ipcRenderer.on('change-SIP', async (event, isSIP = false) => {    

    if(isSIP) {
        if(!window.sip) {
            const sipInstance = new sip.SipClient()       
            await unrg(window.cli)   
            let connDevice = await window.cli.deviceConfig('sip')        
            await sipInstance.register({
                register_sec: 60,
                ...connDevice
            });

            sipInstance.on('registered', async () => {
                console.error("registered");
                window.sip = sipInstance        
                await window.cliController.getCliInstance()
                window.cli.phone = window.sip
            });

            sipInstance.on('unregistered', async () => {
                console.error("Unregistered");
            });
        }        
    }
    else { 
        if(window.sip) {
            await unrg(window.cli)
            window.sip = null
            window.cli.registerCallClient(new SipPhone('electron', true))
        }
               
        await window.cliController.getCliInstance()
    }
})

ipcRenderer.on('call-action', (event, call) => {
    switch (call.action) {
        case 'answer':
            $store.dispatch('call/ANSWER', { callId: call.id })
            break
        case 'reject':
            $store.dispatch('call/HANGUP', { callId: call.id })
            break
        case 'hold':
            $store.dispatch('call/TOGGLE_HOLD', { callId: call.id })
            break
        case 'mute':
            $store.dispatch('call/TOGGLE_MUTE', { callId: call.id })
            break
        default:
            console.error(call)
    }
})

function subscribeAction(store){
    store.subscribeAction((action, state) => {     
        console.log(action.type) 
        switch (action.type) {
            case 'call/HOLD_OTHER_CALLS':
                setActiveCall(action.payload, action.payload?._isOpenedLink? false:true )
                action.payload._isOpenedLink = true
                break
                
            case 'call/TOGGLE_MUTE':
            case 'call/TOGGLE_HOLD':
                updateCall(action.payload.callId, state, action.type)
                break

            case 'workspace/CLOSE_SESSION':
                destroyNotification()
                break
            case 'globals/OPEN_DISCONNECT_POPUP':
                destroyNotification()
                openDisconnectPopup()
                break

            default:
        }
    })
}

function subscribeMutation(store) {
    store.subscribe((mutation, state) => {  
        console.log(mutation.type)      
        switch (mutation.type) {
            case 'status/SET_AGENT_INSTANCE':
                setUserStatus(mutation.payload.status)
                break

            case 'call/SET_CALL_LIST':
                restoreNotification(mutation.payload)
                break
                            
            case 'call/ADD_CALL':          
                setActiveCall(mutation.payload)
                break

            case 'call/REMOVE_CALL':
                if(mutation.payload.id) {
                    removeCallInfo(mutation.payload, state.call.callList)
                }
                break

            default:
        }
    })
}

function updateCall(callId, state, action) {
    let call

    if(callId) call = getCallById(callId, state)
    else call = getCallFromWorkspace(state)

    let ob = createCallObject(call)
    ob.action = action
    
    ipcRenderer.send('update-acive-call', ob)
}

function destroyNotification() {
    ipcRenderer.send('destroy-notification-window')
}

function openDisconnectPopup() {
    ipcRenderer.send('open-disconnect-popup')
}

function restoreNotification(callList) {
    let call = getActiveCall(callList)
    if(call) setActiveCall(call)
}

function setActiveCall(payload, isAnswerIvent) {
    let ob = createCallObject(payload)
    ob.isAnswerIvent = isAnswerIvent
    ipcRenderer.send('set-acive-call', ob)
}

function getActiveCall(callList) {
    let activeCall = callList.find( call => call.isHold === false )
    if(!activeCall)
        activeCall = getFirstCall(callList)
    return activeCall
}

function getFirstCall(callList) {
    if(callList[0].id)
        return callList[0]
    return null
}

function getCallById(callId, state) {
    return state.call.callList.find( call => call.id === callId )
}

function getCallFromWorkspace(state) {
    return state.call.callOnWorkspace
}

function removeCallInfo(payload, callList) {
    let data = {
        removeId: payload.id,
        reserveCall: null
    }
    if(callList.length > 0) {
        let call = getActiveCall(callList)
        data.reserveCall = createCallObject(call)
    }
    ipcRenderer.send('remove-call-info', data)
}

function createCallObject(payload) {
    return {
        allowAnswer: payload.allowAnswer,
        id: payload.id,
        displayName: payload.displayName,
        displayNumber: payload.displayNumber,
        state: payload.state,
        duration: payload.duration,
        direction: payload.direction,
        variables: payload.variables,
        muted: payload.muted,
        isHold: payload.isHold,
        answeredAt: payload.answeredAt,
        queueName: payload.queue?.queue_name,
        answered: payload.answeredAt > 0
    }
}

function setUserStatus(status) {
    ipcRenderer.send('update-tray', status)
}
////////

ipcRenderer.on('reset-timer', (event, arg) => {
    let tasks = $store.state?.status?.agent?.task
    if(tasks){
        let task = tasks.get(arg)
        task.renew()
    }
})

////////////

ipcRenderer.on('set-processing-property', (event, arg = '') => {
    let call = fetCallById(arg.id)
    if(call) {
        switch (arg.prop) {
            case 'isScheduleCall':
                call.postProcessData.isScheduleCall = arg.value
                break
    
            case 'nextDistributeAt':
                call.postProcessData.nextDistributeAt = arg.value
                break
                            
            case 'success':          
                call.postProcessData.success = arg.value
                break
    
            default:
        }
        $store.dispatch('reporting/SET_PROPERTY', arg)
    }
})

ipcRenderer.on('send-reporting', (event, arg) => {
    let call = fetCallById(arg)
    if(call) call.postProcessData.send()
})

ipcRenderer.on('chack-info', (event, arg) => {
    getActiveTask()
})

//setInterval(getActiveTask, 80)
function checkActiveTask(){
    setInterval(getActiveTask, 80)
}

function getActiveTask() {
    if(!$store) return
    let list = $store.state.call.callList
    if(list) {
        let call = list.find(call => call.hangupAt !== 0 && call.allowReporting)
        if(call) {
            setProcessing(call)
        } else {
            ipcRenderer.send('clear-processing')
        }
    }
}

function fetCallById(callId) {
    return $store.state?.call?.callList?.find(call => call.id === callId)
}

function setProcessing(call) {
    let ob = {
        id: call.id,
        taskId: call.task.id,
        state: call.task.state,
        startProcessingAt: call.task.startProcessingAt,
        renewalSec: call.task.renewalSec,
        processingTimeoutAt: call.task.processingTimeoutAt,
        processingSec: call.task.processingSec,
        memberId: call.task.distribute.member_id,
        now: $store.state?.now,
        reporting: {
            isSuccess: call.postProcessData?.success,
            nextDistributeAt: call.postProcessData?.nextDistributeAt,
            isScheduleCall: call.postProcessData?.isScheduleCall
        }
    }

    ipcRenderer.send('update-procesing', ob)
}
